/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "libertine-scope/preview.h"

#include "libertine-scope/i18n.h"
#include "libertine-scope/service_manager.h"
#include <unity/UnityExceptions.h>
#include <unity/scopes/ColumnLayout.h>
#include <unity/scopes/PreviewReply.h>
#include <unity/scopes/Variant.h>
#include <unity/scopes/VariantBuilder.h>
#include <unity/scopes/ActionMetadata.h>
#include <unity/scopes/Result.h>
#include <QDebug>


namespace usc = unity::scopes;


namespace
{
static auto constexpr HEADER_ID = "hdr";
static auto constexpr ACTIONS_ID = "actions";
static auto constexpr METADATA_ID = "other_metadata";
static auto constexpr SUMMARY_ID = "summary";
static auto constexpr SCREENSHOTS_ID = "screenshots";
static auto const SUMMARY_TITLE = _("Info");
static auto const ACTION_INSTALL = _("Install");
static auto const ACTION_REMOVE = _("Remove");
static auto const ACTION_NOOP_REMOVING = _("Removing...");
static auto const ACTION_NOOP_INSTALLING = _("Installing...");
static auto const ACTION_OPEN = _("Open");
static auto const METADATA_PUBLISHER = _("Publisher/Creator");
static auto const METADATA_WEBSITE = _("Website");
static auto const METADATA_LICENSE = _("License");


static usc::PreviewWidget
metadataWidgets(libertine::scope::Package const& app)
{
  usc::PreviewWidget widget(METADATA_ID, "table");
  usc::VariantArray values;
  if (!app.publisher.empty())
  {
    values.push_back(usc::Variant{usc::VariantArray{usc::Variant{METADATA_PUBLISHER}, usc::Variant{app.publisher}}});
  }
  if (!app.website.empty())
  {
    values.push_back(usc::Variant{usc::VariantArray{usc::Variant{METADATA_WEBSITE}, usc::Variant{app.website}}});
  }
  if (!app.license.empty())
  {
    values.push_back(usc::Variant{usc::VariantArray{usc::Variant{METADATA_LICENSE}, usc::Variant{app.license}}});
  }

  widget.add_attribute_value("values", usc::Variant(values));
  return widget;
}


static usc::PreviewWidget
screenshotWidgets(libertine::scope::Package const& app)
{
  usc::PreviewWidget gallery(SCREENSHOTS_ID, "gallery");
  usc::VariantArray arr;
  for (auto const& screenshot: app.screenshots)
  {
    arr.push_back(usc::Variant(screenshot));
  }
  gallery.add_attribute_value("sources", usc::Variant(arr));
  return gallery;
}
}


libertine::scope::Preview::
Preview(usc::Result const&         result,
        usc::ActionMetadata const& metadata,
        std::shared_ptr<ServiceManager>  const& service)
  : PreviewQueryBase(result, metadata)
  , service_(service)
{
}


void libertine::scope::Preview::
cancelled()
{
}


void libertine::scope::Preview::
createSingleColumnLayout(Package const& app)
{
  oneColumn.column.push_back(HEADER_ID);
  oneColumn.column.push_back(ACTIONS_ID);
  oneColumn.column.push_back(METADATA_ID);
  if (!app.screenshots.empty())
  {
    oneColumn.column.push_back(SCREENSHOTS_ID);
  }
  oneColumn.column.push_back(SUMMARY_ID);
}


void libertine::scope::Preview::
createDualColumnLayout(Package const& app)
{
  twoColumns.column1.push_back(HEADER_ID);
  twoColumns.column1.push_back(ACTIONS_ID);
  twoColumns.column1.push_back(SUMMARY_ID);
  twoColumns.column2.push_back(METADATA_ID);
  if (!app.screenshots.empty())
  {
    twoColumns.column2.push_back(SCREENSHOTS_ID);
  }
}


void libertine::scope::Preview::
run(usc::PreviewReplyProxy const& reply)
{
  auto app = service_->app_info(QString::fromStdString(result()["id"].get_string()));
  auto widgets = createWidgets(app);

  createSingleColumnLayout(app);
  createDualColumnLayout(app);

  registerLayouts(reply);
  reply->push(widgets);
}


usc::PreviewWidgetList libertine::scope::Preview::
createWidgets(libertine::scope::Package const& app) const
{
  auto widgets = headerWidgets(app);
  widgets.push_back(buttonWidgets(app));
  widgets.push_back(metadataWidgets(app));
  if (!app.screenshots.empty())
  {
    widgets.push_back(screenshotWidgets(app));
  }
  widgets.push_back(descriptionWidgets(app));

  return widgets;
}


usc::PreviewWidgetList libertine::scope::Preview::
headerWidgets(libertine::scope::Package const& app) const
{
  usc::PreviewWidget header(HEADER_ID, "header");
  header.add_attribute_value("title", usc::Variant(app.name.empty() ? result().title() : app.name));
  header.add_attribute_value("subtitle", usc::Variant(app.summary));
  header.add_attribute_value("mascot", usc::Variant(app.icon.empty() ? result().art() : app.icon));
  header.add_attribute_value("fallback", usc::Variant("image://theme/placeholder-app-icon"));

  usc::PreviewWidgetList widgets;
  widgets.push_back(header);

  qDebug() << "Pushed widgets for package:" << QString::fromStdString(app.name);
  return widgets;
}


usc::PreviewWidget libertine::scope::Preview::
buttonWidgets(libertine::scope::Package const& package) const
{
  if (package.status == "installing" || package.status == "removing")
  {
    usc::PreviewWidget progress(ACTIONS_ID, "progress");
    usc::VariantMap tuple;
    tuple["dbus-name"] = "com.canonical.libertine";
    tuple["dbus-object"] = std::string("/Progress/") + package.progress;
    progress.add_attribute_value("source", usc::Variant(tuple));
    return progress;
  }

  usc::VariantBuilder builder;
  if (package.status == "installed")
  {
    builder.add_tuple(
        {
            {"id", usc::Variant("remove")},
            {"label", usc::Variant(ACTION_REMOVE)},
        });
    builder.add_tuple(
        {
            {"id", usc::Variant("open")},
            {"label", usc::Variant(ACTION_OPEN)},
        });
  }
  else
  {
    builder.add_tuple(
    {
            {"id", usc::Variant("install")},
            {"label", usc::Variant(ACTION_INSTALL)},
          });
  }

  usc::PreviewWidget buttons(ACTIONS_ID, "actions");
  buttons.add_attribute_value("actions", builder.end());
  return buttons;
}


usc::PreviewWidget libertine::scope::Preview::
descriptionWidgets(libertine::scope::Package const& app) const
{
  usc::PreviewWidget summary(SUMMARY_ID, "text");
  summary.add_attribute_value("title", usc::Variant(SUMMARY_TITLE));
  if (app.description.empty() && result().contains("description"))
  {
    summary.add_attribute_value("text", usc::Variant(result()["description"].get_string()));
  }
  else
  {
    summary.add_attribute_value("text", usc::Variant(app.description));
  }
  return summary;
}


void libertine::scope::Preview::
registerLayouts(unity::scopes::PreviewReplyProxy const& reply)
{
    usc::ColumnLayout layout1col(1);
    layout1col.add_column(oneColumn.column);

    usc::ColumnLayout layout2col(2);
    layout2col.add_column(twoColumns.column1);
    layout2col.add_column(twoColumns.column2);

    try
    {
        reply->register_layout({layout1col, layout2col});
    }
    catch (unity::LogicException const& e)
    {
        qWarning() << "Failed to register layout:" << QString::fromStdString(e.what());
    }
}
