/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/store/query.h"

#include <libertine-scope/service_manager.h>
#include <libertine-scope/i18n.h>
#include <unity/scopes/SearchReply.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/VariantBuilder.h>
#include <QtDBus>
#include <QDebug>

namespace usc = unity::scopes;

namespace
{
static constexpr auto CATEGORY_APPS_DISPLAY = R"(
    {
        "schema-version" : 1,
        "template" : {
            "category-layout" : "grid",
            "card-layout" : "horizontal",
            "collapsed-rows": 0,
            "card-size": "large"
        },
        "components" : {
            "title" : "title",
            "art" : {
                "field": "art",
                "aspect-ratio": 1.13,
                "fallback": "image://theme/placeholder-app-icon"
            },
            "subtitle": "subtitle",
            "attributes": { "field": "attributes", "max-count": 3 }
        }
    }
)";


static constexpr auto CATEGORY_HINT = R"(
    {
        "schema-version": 1,
        "template": {
            "category-layout": "grid",
            "card-size": "large",
            "card-layout": "horizontal"
        },
        "components": {
            "title": "title"
        }
    }
)";


static auto const SINGLE_RESULT_FOUND = _("%1 result found");
static auto const MULTIPLE_RESULTS_FOUND = _("%1 results found");
static QString const APPID_URI_FORMAT = "appid://palatine/%1/0.0";
}


std::string const libertine::scope::Query::USE_SEARCH_HINT = _("Use the searchbar to find software.");
std::string const libertine::scope::Query::NO_SEARCH_RESULTS_HINT = _("No results found.");


libertine::scope::Query::
Query(usc::CannedQuery                 const&    query,
      usc::SearchMetadata              const& metadata,
      std::shared_ptr<ServiceManager>  const& service)
  : usc::SearchQueryBase(query, metadata)
  , service_(service)
{
}


void libertine::scope::Query::
cancelled()
{
}


void libertine::scope::Query::
show_hint(usc::SearchReplyProxy const& reply,
          std::string const&           reason) const
{
  auto hint_category = reply->register_category("hint", "", "", usc::CategoryRenderer(CATEGORY_HINT));
  usc::CategorisedResult res(hint_category);
  res.set_uri(usc::CannedQuery(query()).to_uri());
  res.set_title(reason);
  reply->push(res);
}


void libertine::scope::Query::
run_search(usc::SearchReplyProxy const& reply) const
{
  auto results = service_->search_cache(QString::fromStdString(query().query_string()));
  if (results.empty())
  {
    show_hint(reply, NO_SEARCH_RESULTS_HINT);
    return;
  }

  auto category = reply->register_category("aptcache",
        QString::fromStdString(results.size() == 1 ? SINGLE_RESULT_FOUND : MULTIPLE_RESULTS_FOUND)
                               .arg(results.size()).toStdString(),
       "Application",
        usc::CategoryRenderer(CATEGORY_APPS_DISPLAY));

  for (auto const& result: results)
  {
    usc::CategorisedResult cr(category);
    cr.set_title(result.name);
    cr["subtitle"] = result.summary;
    cr["id"] = result.id;
    cr["package"] = result.package;
    cr.set_art(result.icon);
    cr.set_uri(APPID_URI_FORMAT.arg(QString::fromStdString(result.id).remove(".desktop")).toStdString());

    if (!reply->push(cr))
    {
      qCritical() << "Error while adding result " << QString::fromStdString(result.name);
      return;
    }
  }
}


void libertine::scope::Query::
run(usc::SearchReplyProxy const& reply)
{
  if (!query().query_string().empty())
  {
    run_search(reply);
    return;
  }

  show_hint(reply, USE_SEARCH_HINT);
}
