# Copyright 2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from threading import Lock
from libertine import ContainersConfig
from libertine_service import tasks


class SafeContainersConfig(object):
    def __init__(self):
        self.lock = Lock()
        self.config = ContainersConfig.ContainersConfig()
    def __getattr__(self, name):
        config_method = getattr(self.config, name)
        if config_method is not None:
            def _method(*args, **kwargs):
                with self.lock:
                    return config_method(*args)
            return _method


class Container(object):
    def __init__(self, log):
        self.log = log
        self.libertine_config = SafeContainersConfig()
        self.lock = Lock()

    def install(self, package_name, container_id, progress):
        self.libertine_config.refresh_database()
        if not self._container_exists(container_id):
            self.log.debug("Container does not exist, queuing create operation")
            create = tasks.CreateTask(container_id, self.libertine_config, self.lock, self.log)
            create.start()

        task = tasks.InstallTask(package_name, container_id, self.libertine_config, progress, self.lock, self.log)
        return task.start()

    def remove(self, package_name, container_id, progress):
        self.libertine_config.refresh_database()
        if not self._container_exists(container_id):
            self.log.debug("Container does not exist, queuing create operation")
            create = tasks.CreateTask(container_id, self.libertine_config, self.lock, self.log)
            create.start()

        task = tasks.RemoveTask(package_name, container_id, self.libertine_config, progress, self.lock, self.log)
        return task.start()

    def status(self, package_name, container_id):
        self.log.debug("Checking status of package '%s'" % package_name)
        self.libertine_config.refresh_database()
        if not self._container_exists(container_id):
            return ''
        return self.libertine_config.get_package_install_status(container_id, package_name) or ''

    def _container_exists(self, container_id):
        self.log.debug("Checking if container '%s' exists" % container_id)
        return self.libertine_config.container_exists(container_id)
