# Copyright 2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import dbus
import dbus.service
import logging
from libertine_service import apt
from libertine_service import appstream
from libertine_service import container
from dbus.mainloop.glib import DBusGMainLoop
from time import time
import threading


LIBERTINE_SERVICE_NAME = "com.canonical.libertine"
LIBERTINE_STORE_INTERFACE = "com.canonical.libertine.Store"
DOWNLOAD_INTERFACE = "com.canonical.applications.Download"
LIBERTINE_STORE_PATH = "/Store"
CONTAINER_ID = "palatine"

log = logging.getLogger("Libertine")


class Progress(dbus.service.Object):
    def __init__(self, connection):
        log.debug("creating a Progress object")
        self._id = hex(int(time()*10000000))[2:]
        self._finished = False
        dbus.service.Object.__init__(self, conn=connection, object_path=("/Progress/%s" % self._id))

    @property
    def id(self):
        return self._id
    @property
    def done(self):
        return self._finished

    @dbus.service.signal(DOWNLOAD_INTERFACE)
    def finished(self, path):
        log.debug("emit finished('%s')" % path)
        self._finished = True

    @dbus.service.signal(DOWNLOAD_INTERFACE)
    def progress(self, received, total):
        log.debug("emit progress(%d, %d)" % (received, total))


class Service(dbus.service.Object):
    def __init__(self, config):
        log.debug("creating service")
        DBusGMainLoop(set_as_default=True)
        try:
            bus_name = dbus.service.BusName(LIBERTINE_SERVICE_NAME,
                                            bus=dbus.SessionBus(),
                                            do_not_queue=True)
        except dbus.exceptions.NameExistsException:
            log.warning("service is already running")
            raise

        try:
            self.cache = appstream.AppStreamCache(config)
        except:
            log.warning("AppStream backend unavailable, falling back to Apt backend.")
            self.cache = apt.AptCache(config)

        self.container = container.Container(log)
        self.progress = {}

        super().__init__(bus_name, LIBERTINE_STORE_PATH)

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         out_signature='as')
    def get_categories(self):
        log.debug("get_categories() called")
        categories = [ "Fun", "Work", "Pink" ]
        return categories

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         out_signature='as')
    def get_featured(self):
        log.debug("get_featured() called")
        return []

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         out_signature='as')
    def get_popular(self):
        log.debug("get_popular() called")
        return []

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         in_signature='s',
                         out_signature='aa{sv}')
    def search(self, search_string):
        log.debug("search('{}') called".format(search_string))
        return self.cache.search(search_string)

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         in_signature='s',
                         out_signature='a{sv}')
    def app_info(self, app_id):
        log.debug("app_info('{}') called".format(app_id))
        app = self.cache.app_info(app_id)
        if 'package' in app:
            app['status'] = self.container.status(app['package'], CONTAINER_ID)
            if app_id in self.progress:
                app['progress_id'] = self.progress[app_id].id

        return app

    @dbus.service.signal(LIBERTINE_STORE_INTERFACE,
                         signature='su')
    def updating(self, target, percent):
        log.debug("emit updating('{}', {})".format(target, percent))

    # Packaging

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         in_signature='s',
                         out_signature='x')
    def install(self, app_id):
        log.debug("install('%s')" % app_id)
        package = self.cache.app_info(app_id)
        if 'package' in package:
            return self.container.install(package['package'], CONTAINER_ID, self.find_or_create_package_progress(app_id))

        return 0

    @dbus.service.method(LIBERTINE_STORE_INTERFACE,
                         in_signature='s',
                         out_signature='x')
    def remove(self, app_id):
        log.debug("remove('%s')" % app_id)
        package = self.cache.app_info(app_id)
        if 'package' in package:
            return self.container.remove(package['package'], CONTAINER_ID, self.find_or_create_package_progress(app_id))

        return 0

    def find_or_create_package_progress(self, app_id):
        self.cleanup_progress()

        if app_id in self.progress:
            progress = self.progress[app_id]
        else:
            progress = Progress(self.connection)
            self.progress[app_id] = progress

        return progress

    def cleanup_progress(self):
        updated_progress_list = {}
        for key in self.progress.keys():
            if self.progress[key].done:
                self.progress[key].remove_from_connection()
            else:
                updated_progress_list[key] = self.progress[key]
        self.progress = updated_progress_list
