# Copyright 2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from subprocess import Popen, PIPE
from threading import Thread
from abc import ABCMeta, abstractmethod
from libertine import LibertineContainer, HostInfo
from dbus import UInt64


class LibertineTask(metaclass=ABCMeta):
    """
    Abstract class for performing long-running, synchronous operations on a
    Libertine container. Child classes must implement _run, which will execute
    in a separate thread. Override _before to run commands synchronously; if
    _before returns False, _run will not be executed.
    """
    def __init__(self, lock, log):
        self.lock = lock
        self.log = log

    def start(self):
        if self._before():
            thread = Thread(target=self.run)
            thread.start()
            return thread.ident

    def run(self):
        with self.lock:
            self._run()

    @abstractmethod
    def _run(self):
        pass

    def _before(self):
        return True


class InstallTask(LibertineTask):
    def __init__(self, package_name, container_id, config, progress, lock, log):
        super().__init__(lock, log)
        self.package = package_name
        self.container = container_id
        self.config = config
        self.progress = progress
        self.current_progress = 0

    def _run(self):
        self.log.debug("Installing package '%s'" % self.package)
        self._progress()
        container = LibertineContainer(self.container, self.config)
        if container.install_package(self.package):
            self._progress()
            self.config.update_package_install_status(self.container, self.package, "installed")
        else:
            self.log.warning("Package installation failed for '%s'" % self.package)
            self.config.delete_package(self.container, self.package)
        self.progress.finished(self.package)

    def _before(self):
        self.log.debug("InstallTask::_before")
        self._progress()
        self.config.add_new_package(self.container, self.package)
        self.config.update_package_install_status(self.container, self.package, "installing")
        self._progress()
        return True

    def _progress(self):
        self.current_progress += 1
        self.progress.progress(UInt64(self.current_progress), UInt64(self.current_progress+1))


class RemoveTask(LibertineTask):
    def __init__(self, package_name, container_id, config, progress, lock, log):
        super().__init__(lock, log)
        self.package = package_name
        self.container = container_id
        self.config = config
        self.progress = progress
        self.current_progress = 0
        self.fallback = "installed"

    def _run(self):
        self.log.debug("Removing package '%s'" % self.package)
        self._progress()
        container = LibertineContainer(self.container, self.config)
        if container.remove_package(self.package):
            self._progress()
            self.config.delete_package(self.container, self.package)
        else:
            self.log.warning("Package removal failed for '%s'" % self.package)
            self.config.update_package_install_status(self.container, self.package, self.fallback)
        self.progress.finished(self.package)

    def _before(self):
        self.log.debug("RemoveTask::_before")
        self._progress()
        if self.config.package_exists(self.container, self.package):
            self.fallback = self.config.get_package_install_status(self.container, self.package)
            self.config.update_package_install_status(self.container, self.package, "removing")
            self._progress()
            return True
        else:
            self.log.debug("Package '%s' already exists, skipping install" % self.package)
            return False

    def _progress(self):
        self.current_progress += 1
        self.progress.progress(UInt64(self.current_progress), UInt64(self.current_progress+1))


class CreateTask(LibertineTask):
    def __init__(self, container_id, config, lock, log):
        super().__init__(lock, log)
        self.container = container_id
        self.config = config

    def _run(self):
        self.log.debug("Creating container '%s'" % self.container)
        container = LibertineContainer(self.container, self.config)
        if not container.create_libertine_container(password=''):
            self.log.debug("Creating container '%s' failed" % self.container)
            self.config.delete_container(self.container)
            return
        self.config.update_container_install_status(self.container, "ready")

    def _before(self):
        self.log.debug("CreateTask::_before")
        if self.config.container_exists(self.container):
            return False

        info = HostInfo.HostInfo()

        self.config.add_new_container(self.container, 'User Applications', info.select_container_type_by_kernel(),
                                      info.get_host_distro_release())
        self.config.update_container_install_status(self.container, 'installing')
        return True
