/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/apps/preview.h"
#include "tests/scope/apps/mock_hidden_apps.h"

#include <gtest/gtest.h>
#include <memory>
#include <unity/scopes/Variant.h>
#include <unity/scopes/ActionMetadata.h>
#include <unity/scopes/testing/Result.h>
#include <unity/scopes/testing/MockPreviewReply.h>
#include <unity/scopes/PreviewWidget.h>
#include <tests/lib/libertine-scope/mock_service_manager.h>


namespace
{
static void
verifyHeader(libertine::scope::Package const& package, unity::scopes::PreviewWidget const& header)
{
  EXPECT_EQ("hdr", header.id());
  EXPECT_EQ("header", header.widget_type());

  EXPECT_EQ(package.name, header.attribute_values()["title"].get_string());
  EXPECT_EQ(package.summary, header.attribute_values()["subtitle"].get_string());
  EXPECT_EQ(package.icon, header.attribute_values()["mascot"].get_string());
}


static void
verifyHiddenButtons(unity::scopes::PreviewWidget const& buttons)
{
  EXPECT_EQ("actions", buttons.id());
  EXPECT_EQ("actions", buttons.widget_type());
  auto buttons_actions = buttons.attribute_values()["actions"].get_array();
  ASSERT_EQ(2, buttons_actions.size());
  EXPECT_EQ("open", buttons_actions[0].get_dict()["id"].get_string());
  EXPECT_EQ("Open", buttons_actions[0].get_dict()["label"].get_string());
  EXPECT_EQ("show", buttons_actions[1].get_dict()["id"].get_string());
  EXPECT_EQ("Show", buttons_actions[1].get_dict()["label"].get_string());
}


static void
verifyShownButtons(unity::scopes::PreviewWidget const& buttons)
{
  EXPECT_EQ("actions", buttons.id());
  EXPECT_EQ("actions", buttons.widget_type());
  auto buttons_actions = buttons.attribute_values()["actions"].get_array();
  ASSERT_EQ(2, buttons_actions.size());
  EXPECT_EQ("open", buttons_actions[0].get_dict()["id"].get_string());
  EXPECT_EQ("Open", buttons_actions[0].get_dict()["label"].get_string());
  EXPECT_EQ("hide", buttons_actions[1].get_dict()["id"].get_string());
  EXPECT_EQ("Hide", buttons_actions[1].get_dict()["label"].get_string());
}


static void
verifyMetadata(libertine::scope::Package const& package, unity::scopes::PreviewWidget const& props)
{
  EXPECT_EQ("other_metadata", props.id());
  EXPECT_EQ("table", props.widget_type());

  auto propValues = props.attribute_values()["values"].get_array();
  ASSERT_EQ(3, propValues.size());
  ASSERT_EQ(2, propValues[0].get_array().size());
  EXPECT_EQ("Publisher/Creator", propValues[0].get_array()[0].get_string());
  EXPECT_EQ(package.publisher, propValues[0].get_array()[1].get_string());
  ASSERT_EQ(2, propValues[1].get_array().size());
  EXPECT_EQ("Website", propValues[1].get_array()[0].get_string());
  EXPECT_EQ(package.website, propValues[1].get_array()[1].get_string());
  ASSERT_EQ(2, propValues[2].get_array().size());
  EXPECT_EQ("License", propValues[2].get_array()[0].get_string());
  EXPECT_EQ(package.license, propValues[2].get_array()[1].get_string());
}


static void
verifyDescription(libertine::scope::Package const& package, unity::scopes::PreviewWidget const& description)
{
  EXPECT_EQ("summary", description.id());
  EXPECT_EQ("text", description.widget_type());
  EXPECT_EQ("Info", description.attribute_values()["title"].get_string());
  EXPECT_EQ(package.description, description.attribute_values()["text"].get_string());
}


static void
verifyScreenshots(libertine::scope::Package const& package, unity::scopes::PreviewWidget const& screenshots)
{
  EXPECT_EQ("screenshots", screenshots.id());
  EXPECT_EQ("gallery", screenshots.widget_type());

  auto sources = screenshots.attribute_values()["sources"].get_array();
  ASSERT_EQ(package.screenshots.size(), sources.size());

  for (auto i = 0u; i < package.screenshots.size(); ++i)
  {
    EXPECT_EQ(package.screenshots[i], sources[i].get_string());
  }

}


static libertine::scope::Package
basePackage()
{
  libertine::scope::Package package;
  package.name = "vim";
  package.summary = "classic text editor";
  package.description = "this or emacs";
  package.icon = "file:///test/file.png";
  package.publisher = "Woody Boyd";
  package.website = "http://canonical.com/";
  package.license = "GPLv8+";
  package.id = "vim.desktop";
  package.status = "";
  package.package = "vim-common";
  return package;
}


class AppsPreviewTest : public testing::Test
{
protected:
  AppsPreviewTest()
    : testing::Test()
    , result()
    , metadata("en_US", "phone")
    , list(new unity::scopes::PreviewWidgetList())
    , reply()
    , proxy(&reply, [](unity::scopes::PreviewReply*) {})
    , service(new testing::NiceMock<libertine::scope::MockServiceManager>())
    , hidden(new testing::NiceMock<MockHiddenApps>())
  {
  }

  virtual void
  SetUp()
  {
    EXPECT_CALL(reply, push(testing::_)).WillOnce(testing::SaveArg<0>(list.get()));
  }

  void usePackage(libertine::scope::Package const& package)
  {
    result["id"] = unity::scopes::Variant(package.id);
    result["app_id"] = unity::scopes::Variant(package.id + ".desktop");
    EXPECT_CALL(*service, app_info(QString::fromStdString(package.id))).WillOnce(testing::Return(package));
  }

  unity::scopes::testing::Result result;
  unity::scopes::ActionMetadata metadata;
  std::unique_ptr<unity::scopes::PreviewWidgetList> list;
  testing::NiceMock<unity::scopes::testing::MockPreviewReply> reply;
  unity::scopes::PreviewReplyProxy proxy;
  std::shared_ptr<testing::NiceMock<libertine::scope::MockServiceManager> > service;
  std::shared_ptr<testing::NiceMock<MockHiddenApps> > hidden;
};


TEST_F(AppsPreviewTest, ShowsDetailedInformationForPackage)
{
  auto package = basePackage();
  usePackage(package);

  EXPECT_CALL(*hidden, app_is_hidden(testing::_)).WillOnce(testing::Return(false));

  Preview preview(result, metadata, service, hidden);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(4, list->size());

  verifyHeader(package, list->front());
  list->pop_front();
  verifyShownButtons(list->front());
  list->pop_front();
  verifyMetadata(package, list->front());
  list->pop_front();
  verifyDescription(package, list->front());
}


TEST_F(AppsPreviewTest, ShowsShowButtonsForHiddenPackage)
{
  auto package = basePackage();
  usePackage(package);

  EXPECT_CALL(*hidden, app_is_hidden(testing::_)).WillOnce(testing::Return(true));

  Preview preview(result, metadata, service, hidden);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(4, list->size());

  verifyHeader(package, list->front());
  list->pop_front();
  verifyHiddenButtons(list->front());
  list->pop_front();
  verifyMetadata(package, list->front());
  list->pop_front();
  verifyDescription(package, list->front());
}


TEST_F(AppsPreviewTest, ShowsInfoFromResultWhenPackageIsEmpty)
{
  libertine::scope::Package package;
  package.id = "vim-common";
  usePackage(package);
  result["description"] = unity::scopes::Variant("This is a summary of vim.");
  result.set_art("file:///some/cool/image.png");
  result.set_title("VIM");

  EXPECT_CALL(*hidden, app_is_hidden(testing::_)).WillOnce(testing::Return(false));

  Preview preview(result, metadata, service, hidden);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(4, list->size());

  auto header = list->front();
  EXPECT_EQ("hdr", header.id());
  EXPECT_EQ("header", header.widget_type());

  EXPECT_EQ(result.title(), header.attribute_values()["title"].get_string());
  EXPECT_TRUE(header.attribute_values()["subtitle"].get_string().empty());
  EXPECT_EQ(result.art(), header.attribute_values()["mascot"].get_string());

  list->pop_front();
  verifyShownButtons(list->front());
  list->pop_front();
  list->pop_front();

  auto description = list->front();
  EXPECT_EQ("summary", description.id());
  EXPECT_EQ("text", description.widget_type());
  EXPECT_EQ("Info", description.attribute_values()["title"].get_string());
  EXPECT_EQ(result["description"].get_string(), description.attribute_values()["text"].get_string());
}


TEST_F(AppsPreviewTest, ShowsScreenshotsWhenAvailable)
{
  auto package = basePackage();
  package.screenshots = std::vector<std::string>{"file:///some/string/uri.png", "file:///some/other/string/uri.jpg"};
  usePackage(package);

  EXPECT_CALL(*hidden, app_is_hidden(testing::_)).WillOnce(testing::Return(false));

  Preview preview(result, metadata, service, hidden);
  preview.run(proxy);

  ASSERT_NE(nullptr, list);
  ASSERT_EQ(5, list->size());

  verifyHeader(package, list->front());
  list->pop_front();
  verifyShownButtons(list->front());
  list->pop_front();
  verifyMetadata(package, list->front());
  list->pop_front();
  verifyScreenshots(package, list->front());
  list->pop_front();
  verifyDescription(package, list->front());
}
}
