/*
 * Copyright 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3, as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scope/store/query.h"


#include "tests/lib/libertine-scope/mock_service_manager.h"
#include <unity/scopes/SearchMetadata.h>
#include <unity/scopes/CannedQuery.h>
#include <unity/scopes/testing/MockSearchReply.h>
#include <unity/scopes/Category.h>
#include <memory>
#include <gtest/gtest.h>
#include <gmock/gmock.h>


namespace
{
using namespace libertine::scope;


class FakeCategory : public unity::scopes::Category
{
public:
    FakeCategory(std::string const& id, std::string const& title,
                 std::string const& icon, unity::scopes::CategoryRenderer const& renderer) :
       unity::scopes::Category(id, title, icon, renderer)
    {
    }
};


MATCHER_P(ResultTitleMatch, title, "")
{
  return arg.contains("title") && arg["title"] == unity::scopes::Variant(title);
}


MATCHER_P4(ResultPackageMatch, title, subtitle, id, package, "")
{
  return arg.contains("title") && arg["title"].get_string() == title &&
      arg.contains("subtitle") && arg["subtitle"].get_string() == subtitle &&
      arg.contains("id") && arg["id"].get_string() == id &&
      arg.contains("package") && arg["package"].get_string() == package;
}


class TestStoreQuery : public ::testing::Test
{
protected:
  TestStoreQuery()
    : metadata_("en_US", "phone")
    , canned_query_("libertine-scope")
    , reply_()
    , proxy_(&reply_, [](unity::scopes::SearchReply*) {})
    , category_(std::make_shared<FakeCategory>("fake-container", "fake-container", "Application", unity::scopes::CategoryRenderer()))
    , service_(new testing::NiceMock<MockServiceManager>())
  {
  }

  Package create_package(std::string const& name, std::string const& summary, std::string const& id, std::string const& pkgname)
  {
    Package package;
    package.name = name;
    package.summary = summary;
    package.package = pkgname;
    package.id = id;

    return package;
  }

  void expect_push(std::string const& title, std::string const& subtitle, std::string const& id, std::string const& package,
                   bool result = true)
  {
    EXPECT_CALL(reply_, push(testing::Matcher<unity::scopes::CategorisedResult const&>(ResultPackageMatch(title, subtitle, id, package)))).WillOnce(testing::Return(result));
  }

  unity::scopes::SearchMetadata metadata_;
  unity::scopes::CannedQuery canned_query_;
  testing::NiceMock<unity::scopes::testing::MockSearchReply> reply_;
  unity::scopes::SearchReplyProxy proxy_;
  std::shared_ptr<FakeCategory> category_;
  std::shared_ptr<testing::NiceMock<MockServiceManager>> service_;
};


TEST_F(TestStoreQuery, ShowsSearchHintOnEmptyQuery)
{
  EXPECT_CALL(reply_, register_category("hint", "", "", testing::_)).WillOnce(testing::Return(category_));
  EXPECT_CALL(reply_, push(testing::Matcher<unity::scopes::CategorisedResult const&>(ResultTitleMatch(Query::USE_SEARCH_HINT)))).WillOnce(testing::Return(true));

  Query q(canned_query_, metadata_, service_);
  q.run(proxy_);
}


TEST_F(TestStoreQuery, ShowsNoResultsHintOnQueryWithNoResults)
{
  EXPECT_CALL(reply_, register_category("hint", "", "", testing::_)).WillOnce(testing::Return(category_));
  EXPECT_CALL(reply_, push(testing::Matcher<unity::scopes::CategorisedResult const&>(ResultTitleMatch(Query::NO_SEARCH_RESULTS_HINT)))).WillOnce(testing::Return(true));
  EXPECT_CALL(*service_, search_cache(testing::_)).WillOnce(testing::Return(QList<Package>{}));
  canned_query_.set_query_string("foobar");

  Query q(canned_query_, metadata_, service_);
  q.run(proxy_);
}


TEST_F(TestStoreQuery, ShowSingleSearchResult)
{
  EXPECT_CALL(reply_, register_category("aptcache", "1 result found", "Application", testing::_)).WillOnce(testing::Return(category_));

  auto package = create_package("vim", "Writin' stuff", "vim.desktop", "vim-common");
  expect_push("vim", "Writin' stuff", "vim.desktop", "vim-common");

  EXPECT_CALL(*service_, search_cache(testing::_)).WillOnce(testing::Return(QList<Package>{package}));
  canned_query_.set_query_string("vim");

  Query q(canned_query_, metadata_, service_);
  q.run(proxy_);
}


TEST_F(TestStoreQuery, ShowMultipleSearchResults)
{
  EXPECT_CALL(reply_, register_category("aptcache", "3 results found", "Application", testing::_)).WillOnce(testing::Return(category_));

  auto vim = create_package("vim", "Writin' stuff", "vim.desktop", "vim-common");
  auto gedit = create_package("gedit", "Writin' gui stuff", "gedit.desktop", "gedit");
  auto ed = create_package("ed", "Writin' simple stuff", "ed.desktop", "ed");

  expect_push("vim", "Writin' stuff", "vim.desktop", "vim-common");
  expect_push("gedit", "Writin' gui stuff", "gedit.desktop", "gedit");
  expect_push("ed", "Writin' simple stuff", "ed.desktop", "ed");

  EXPECT_CALL(*service_, search_cache(testing::_)).WillOnce(testing::Return(QList<Package>{vim, gedit, ed}));
  canned_query_.set_query_string("vim");

  Query q(canned_query_, metadata_, service_);
  q.run(proxy_);
}


TEST_F(TestStoreQuery, FailureToPushEndsQueryPrematurely)
{
  EXPECT_CALL(reply_, register_category("aptcache", "3 results found", "Application", testing::_)).WillOnce(testing::Return(category_));

  auto vim = create_package("vim", "Writin' stuff", "vim.desktop", "vim-common");
  auto gedit = create_package("gedit", "Writin' gui stuff", "gedit.desktop", "gedit");
  auto ed = create_package("ed", "Writin' simple stuff", "ed.desktop", "ed");

  expect_push("vim", "Writin' stuff", "vim.desktop", "vim-common");
  expect_push("gedit", "Writin' gui stuff", "gedit.desktop", "gedit", false);

  EXPECT_CALL(*service_, search_cache(testing::_)).WillOnce(testing::Return(QList<Package>{vim, gedit, ed}));
  canned_query_.set_query_string("vim");

  Query q(canned_query_, metadata_, service_);
  q.run(proxy_);
}
}
